﻿//PA Module
define("PA", [], function () {
    return function () {
        this.componentid = "";
        this.accounts = [];
        this.benchmarks = [];
        this.dates = {};
        this.groups = null;
        this.columns = null;
        this.currencyisocode = null;

        this.documentName = "PA3_DOCUMENTS:DEFAULT";
        this.isdocumentLookupSuccess = false;
        this.componentsList = [];
        this.isComponentSelected = false;
        this.isDatesSelected = false;

        this.hasDefaultAccounts = false;
        this.hasDefaultBenchmarks = false;

        this.isAccountExist = function (accountId, holdingsMode) {
            return this.accounts.map(function (account) {
                if (account.id.toUpperCase() === accountId.toUpperCase()) {
                    account.holdingsmode ? account.holdingsmode = account.holdingsmode.toUpperCase() : account.holdingsmode = null;
                    holdingsMode ? holdingsMode = holdingsMode.toUpperCase() : holdingsMode = null;
                    if (account.holdingsmode === holdingsMode) {
                        return true;
                    }
                    else
                        return false;
                } else {
                    return false;
                }
            }).filter(m => m === true).length > 0;
        };

        this.isBenchmarkExist = function (benchmarkId, holdingsMode) {
            return this.benchmarks.map(function (benchmark) {
                if (benchmark.id.toUpperCase() === benchmarkId.toUpperCase()) {
                    benchmark.holdingsmode ? benchmark.holdingsmode = benchmark.holdingsmode.toUpperCase() : benchmark.holdingsmode = null;
                    holdingsMode ? holdingsMode = holdingsMode.toUpperCase() : holdingsMode = null;

                    if (benchmark.holdingsmode === holdingsMode)
                        return true;
                    else
                        return false;
                } else {
                    return false;
                }
            }).filter(m => m === true).length > 0;
        };

        this.addAccount = function (accountInput, accountHoldingsMode) {
            if (accountInput) {
                var accountArray = accountInput.split(',');
                accountArray.forEach((account) => {
                    account = account.trim().toUpperCase();
                    var isAccExist = this.isAccountExist(account, accountHoldingsMode);
                    if (!isAccExist)
                        this.accounts.push({ id: account, holdingsmode: accountHoldingsMode });
                });
            }
        };

        this.addBenchmark = function (benchmarkInput, benchmarkHoldingsMode) {
            if (benchmarkInput) {
                var benchmarkArray = benchmarkInput.split(',');
                benchmarkArray.forEach((benchmark) => {
                    benchmark = benchmark.trim().toUpperCase();
                    var isBenchExist = this.isBenchmarkExist(benchmark, benchmarkHoldingsMode);
                    if (!isBenchExist)
                        this.benchmarks.push({ id: benchmark, holdingsmode: benchmarkHoldingsMode });
                });
            }
        };

        this.removeAccount = function (account) {
            this.accounts = this.accounts.filter(y => !(y.id === account.id && y.holdingsmode === account.holdingsmode));
        };

        this.removeBenchmark = function (benchmark) {
            this.benchmarks = this.benchmarks.filter(y => !(y.id === benchmark.id && y.holdingsmode === benchmark.holdingsmode));
        };

        this.validate = function () {
            var isValid = true;
            var errorMsg = [];

            if (!this.componentid) {
                isValid = false;
                errorMsg.push("Enter valid component id");
            }
            if (this.accounts.length === 0) {
                isValid = false;
                errorMsg.push("Enter atleast one account");
            }

            return { valid: isValid, error: errorMsg.join("\\n") };

        };

        // Build the PA post body object
        this.GetCalculationUnitParameters = function () {
            var pa = {};

            pa.accounts = this.accounts;
            pa.benchmarks = this.benchmarks.length !== 0 ? this.benchmarks : null;
            pa.componentid = this.componentid;

            if (this.dates.startdate || this.dates.enddate || this.dates.frequency) {
                pa.dates = {};
                pa.dates.startdate = this.dates.startdate || "";
                pa.dates.enddate = this.dates.enddate || "";
                pa.dates.frequency = this.dates.frequency || "";
            }

            if (this.groups !== null) {
                pa.groups = this.groups;
            }
            if (this.columns !== null) {
                pa.columns = this.columns;
            }
            if (this.currencyisocode !== null) {
                pa.currencyisocode = this.currencyisocode;
            }
            return pa;
        };
    };
});

//SPAR Module
define("SPAR", [], function () {
    return function () {
        this.componentid = "";
        this.accounts = [];
        this.benchmark = {};
        this.dates = {};

        this.documentName = "SPAR_DOCUMENTS:Factset Default Document";
        this.isdocumentLookupSuccess = false;
        this.componentsList = [];
        this.isComponentSelected = false;
        this.isDatesSelected = false;

        this.isAccountExist = function (id, returnType, prefix) {
            return this.accounts.map(function (account) {
                if (account.id.toUpperCase() === id.toUpperCase()) {

                    account.returntype = account.returntype ? account.returntype.toUpperCase() : null;
                    account.prefix = account.prefix ? account.prefix.toUpperCase() : null;
                    returnType = returnType ? returnType.toUpperCase() : null;
                    prefix = prefix ? prefix.toUpperCase() : null;

                    if (account.returnType !== returnType) {
                        return false;
                    }

                    if (account.prefix !== prefix) {
                        return false;
                    }
                    return true;
                } else {
                    return false;
                }
            }).filter(m => m === true).length > 0;
        };

        this.addAccount = function (id, returnType, prefix) {
            if (id) {
                var isAccExist = this.isAccountExist(id, returnType, prefix);
                if (!isAccExist) {
                    var acct = {};
                    acct.id = id.toUpperCase();
                    acct.returntype = returnType ? returnType.toUpperCase() : null;
                    acct.prefix = prefix ? prefix.toUpperCase() : null;
                    this.accounts.push(acct);
                }
            }
        };

        this.addBenchmark = function (id, returnType, prefix) {
            if (id) {
                this.benchmark = {};
                this.benchmark.id = id.toUpperCase();
                this.benchmark.returntype = returnType ? returnType.toUpperCase() : null;
                this.benchmark.prefix = prefix ? prefix.toUpperCase() : null;
            }
        };

        this.removeAccount = function (account) {
            this.accounts = this.accounts.filter(y => !(y.id === account.id && y.prefix === account.prefix && y.returntype === account.returntype));
        };

        this.validate = function () {
            var isValid = true;
            var errorMsg = [];

            if (this.componentid === "") {
                isValid = false;
                errorMsg.push("Enter valid component id");
            }

            if (this.accounts.length === 0) {
                isValid = false;
                errorMsg.push("Enter atleast one account");
            }

            return { valid: isValid, error: errorMsg.join("\\n") };
        };

        this.GetCalculationUnitParameters = function () {
            var spar = {};
            spar.componentid = this.componentid;
            spar.accounts = this.accounts;
            spar.benchmark = this.benchmark.id ? this.benchmark : null;

            if (this.dates.startdate || this.dates.enddate || this.dates.frequency) {
                spar.dates = {};
                spar.dates.startdate = this.dates.startdate || "";
                spar.dates.enddate = this.dates.enddate || "";
                spar.dates.frequency = this.dates.frequency || "";
            }

            return spar;
        };
    };
});

//Vault Module
define("Vault", [], function () {
    return function () {
        this.componentid = "";
        this.configid = "";
        this.account = {};
        this.dates = {};

        this.documentName = "PA3_DOCUMENTS:DEFAULT";
        this.isdocumentLookupSuccess = false;
        this.componentsList = [];
        this.isComponentSelected = false;
        this.isDatesSelected = false;

        this.hasDefaultAccount = false;
        this.hasDefaultBenchmark = false;

        this.validate = function () {
            var isValid = true;
            var errorMsg = [];

            if (!this.componentid) {
                isValid = false;
                errorMsg.push("Enter valid component id");
            }

            if (!this.configid) {
                isValid = false;
                errorMsg.push("Enter configuration id");
            }

            if (!this.account.id) {
                isValid = false;
                errorMsg.push("Enter account");
            }

            return { valid: isValid, error: errorMsg.join(", ") };
        };

        this.GetCalculationUnitParameters = function () {
            var vault = {};
            vault.componentid = this.componentid;
            vault.configid = this.configid;
            vault.account = this.account;
            if (this.dates.startdate || this.dates.enddate || this.dates.frequency) {
                vault.dates = {};
                vault.dates.startdate = this.dates.startdate || "";
                vault.dates.enddate = this.dates.enddate || "";
                vault.dates.frequency = this.dates.frequency || "";
            }
            return vault;
        };
    };
});

define(['qvangular', "PA", "SPAR", "Vault",
    'text!./connectdialog.ng.html',
    'css!./connectdialog.css'], function (qvangular, PA, SPAR, Vault, template) {
        return {
            template: template,
            controller: ['$scope', 'input', function ($scope, input) {

                function init() {
                    $scope.name = "";

                    var apiAuthCookie = {};
                    var apiAuthCookieString = document.cookie.split(';').filter(x => x.startsWith("apiAuthentication"))[0];
                    if (apiAuthCookieString) {
                        apiAuthCookie = JSON.parse(apiAuthCookieString.replace("apiAuthentication=", ""));
                    }

                    $scope.username = apiAuthCookie.username || "";
                    $scope.serial = apiAuthCookie.serial || "";
                    $scope.password = apiAuthCookie.password || "";

                    $scope.showLoader = false;
                    $scope.pa = new PA();
                    $scope.spar = new SPAR();
                    $scope.vault = new Vault();

                    $scope.paArray = [];
                    $scope.sparArray = [];
                    $scope.vaultArray = [];

                    $scope.paFrequencies = [];
                    $scope.sparFrequencies = [];
                    $scope.vaultFrequencies = [];

                    $scope.paAccountHoldingsmode = "B&H";
                    $scope.paBenchmarkHoldingsmode = "B&H";

                    $scope.paId = 1;
                    $scope.sparId = 1;
                    $scope.vaultId = 1;

                    $scope.pa_error = "";
                    $scope.spar_error = "";
                    $scope.vault_error = "";

                    $scope.isEdit = input.editMode;
                    $scope.id = input.instanceId;
                    $scope.isEditPostBody = false;

                    $scope.isValidCredentials = false;
                    $scope.postbody = "";

                    $scope.provider = "FactSetQlikConnector.exe";
                    $scope.connectionInfo = "";
                    $scope.tablesMetadata = [];
                    $scope.tablesMetadataString = "";
                    $scope.resultLocations = [];
                    $scope.resultLocationsString = "";

                    $scope.batchHasFailureCalcs = false;
                    $scope.batchFailuresSummary = "";
                    $scope.metadataResponseOnPartialSuccess = null;
                }

                //** Helper Methods - Start **
                $scope.getUsernameSerial = function () {
                    return $scope.username + "-" + $scope.serial;
                };

                $scope.getCalculationPostBody = function () {
                    var post = {};

                    if ($scope.paArray.length !== 0) {
                        post.pa = {};
                        $scope.paArray.forEach(paObj => {
                            post.pa[paObj.id] = paObj.pa.GetCalculationUnitParameters();
                        });
                    }

                    if ($scope.sparArray.length !== 0) {
                        post.spar = {};
                        $scope.sparArray.forEach(sparObj => {
                            post.spar[sparObj.id] = sparObj.spar.GetCalculationUnitParameters();
                        });
                    }

                    if ($scope.vaultArray.length !== 0) {
                        post.vault = {};
                        $scope.vaultArray.forEach(vaultObj => {
                            post.vault[vaultObj.id] = vaultObj.vault.GetCalculationUnitParameters();
                        });
                    }

                    $scope.postbody = JSON.stringify(post).replace(/\"/g, "\'");
                };

                // builds the connection using the $scope parameters provider, username, serial, password, postbody, 
                // tablesMetadataString and resultLocationsString.
                $scope.buildConnectionString = function () {
                    var conString = "CUSTOM CONNECT TO \"provider=" + $scope.provider + ";apiusernameserial=" + $scope.username + "-" + $scope.serial + ";apipassword=" + $scope.password + ";postBody=" + $scope.postbody + ";tablesMetadata=" + $scope.tablesMetadataString + ";resultLocations=" + $scope.resultLocationsString + ";\"";
                    return conString;
                };
                //** Helper Methods - End **


                //** sendJsonRequest call methods - Start **    
                $scope.validateCredentials = function () {
                    return input.serverside.sendJsonRequest("validateCredentials", $scope.getUsernameSerial(), $scope.password).then((response) => {
                        $scope.connectionInfo = "";
                        return JSON.parse(response.qMessage);
                    });
                };

                $scope.getPAComponents = function () {
                    return input.serverside.sendJsonRequest("getPAComponents", $scope.getUsernameSerial(), $scope.password, $scope.pa.documentName).then((response) => {
                        return JSON.parse(response.qMessage);
                    });
                };

                $scope.getSPARComponents = function () {
                    return input.serverside.sendJsonRequest("getSPARComponents", $scope.getUsernameSerial(), $scope.password, $scope.spar.documentName).then((response) => {
                        return JSON.parse(response.qMessage);
                    });
                };

                $scope.getVaultComponents = function () {
                    return input.serverside.sendJsonRequest("getVaultComponents", $scope.getUsernameSerial(), $scope.password, $scope.vault.documentName).then((response) => {
                        return JSON.parse(response.qMessage);
                    });
                };

                $scope.getPAFrequencies = function () {
                    return input.serverside.sendJsonRequest("getPAFrequencies", $scope.getUsernameSerial(), $scope.password).then((response) => {
                        return JSON.parse(response.qMessage);
                    });
                };

                $scope.getSPARFrequencies = function () {
                    return input.serverside.sendJsonRequest("getSPARFrequencies", $scope.getUsernameSerial(), $scope.password).then((response) => {
                        return JSON.parse(response.qMessage);
                    });
                };

                $scope.getVaultFrequencies = function () {
                    return input.serverside.sendJsonRequest("getVaultFrequencies", $scope.getUsernameSerial(), $scope.password).then((response) => {
                        return JSON.parse(response.qMessage);
                    });
                };

                $scope.getVaultConfigurations = function () {
                    return input.serverside.sendJsonRequest("getVaultConfigurations", $scope.getUsernameSerial(), $scope.password, $scope.vault.account.id).then((response) => {
                        return JSON.parse(response.qMessage);
                    });
                };

                $scope.getPAComponenetDetails = function () {
                    return input.serverside.sendJsonRequest("getPAComponentDetails", $scope.getUsernameSerial(), $scope.password, $scope.pa.componentid).then((response) => {
                        return JSON.parse(response.qMessage);
                    });
                };

                $scope.getVaultComponenetDetails = function () {
                    return input.serverside.sendJsonRequest("getVaultComponentDetails", $scope.getUsernameSerial(), $scope.password, $scope.vault.componentid).then((response) => {
                        return JSON.parse(response.qMessage);
                    });
                };

                $scope.getSPARComponenetDetails = function () {
                    return input.serverside.sendJsonRequest("getSPARComponentDetails", $scope.getUsernameSerial(), $scope.password, $scope.spar.componentid).then((response) => {
                        return JSON.parse(response.qMessage);
                    });
                };

                $scope.getTablesMetada = function () {
                    return input.serverside.sendJsonRequest("getTablesMetada", $scope.getUsernameSerial(), $scope.password, $scope.postbody).then((response) => {
                        return JSON.parse(response.qMessage);
                    });
                };
                //** sendJsonRequest call methods - End **    


                //** clearing error messages - Start **  
                $scope.clearPAError = function () {
                    $scope.pa_error = "";
                };

                $scope.clearVaultError = function () {
                    $scope.vault_error = "";
                };

                $scope.clearSPARError = function () {
                    $scope.spar_error = "";
                };

                $scope.clearConnectionError = function () {
                    $scope.connectionInfo = "";
                };
                //** clearing error messages - End **  


                //** Adding/Removing individual calculations to array - Start **  
                $scope.addPAtoArray = function () {
                    var output = $scope.pa.validate();
                    if (output.valid) {
                        $scope.pa_error = "";
                        var paObj = {};
                        paObj.id = "PA" + $scope.paId;
                        paObj.pa = $scope.pa;
                        $scope.paArray.push(paObj);
                        $scope.paId++;
                        $scope.pa = new PA();
                    } else {
                        $scope.pa_error = output.error;
                    }
                };

                $scope.addSPARtoArray = function () {
                    var output = $scope.spar.validate();
                    if (output.valid) {
                        $scope.spar_error = "";
                        var sparObj = {};
                        sparObj.id = "SPAR" + $scope.sparId;
                        sparObj.spar = $scope.spar;
                        $scope.sparArray.push(sparObj);
                        $scope.sparId++;
                        $scope.spar = new SPAR();
                    } else {
                        $scope.spar_error = output.error;
                    }
                };

                $scope.addVaulttoArray = function () {
                    var output = $scope.vault.validate();
                    if (output.valid) {
                        $scope.vault_error = "";
                        var vaultObj = {};
                        vaultObj.id = "VAULT" + $scope.vaultId;
                        vaultObj.vault = $scope.vault;
                        $scope.vaultArray.push(vaultObj);
                        $scope.vaultId++;
                        $scope.vault = new Vault();
                    } else {
                        $scope.vault_error = output.error;
                    }
                };

                $scope.removeCalculation = function (engine, id) {
                    if (engine === 'pa') {
                        $scope.paArray = $scope.paArray.filter(y => y.id !== id);
                    }
                    if (engine === 'spar') {
                        $scope.sparArray = $scope.sparArray.filter(y => y.id !== id);
                    }
                    if (engine === 'vault') {
                        $scope.vaultArray = $scope.vaultArray.filter(y => y.id !== id);
                    }
                };
                //** Adding/Removing individual calculations to array - End **  


                //** Event handlers - Start ** 
                $scope.onValidateCredentialsClicked = function () {
                    if ($scope.name === "" || $scope.name === null) {
                        $scope.connectionInfo = "Please enter a name for the connection.";
                    } else {
                        $scope.showLoader = true;

                        $scope.validateCredentials().then((apiResponse) => {
                            $scope.showLoader = false;

                            if (apiResponse.status === "Success") {
                                $scope.isValidCredentials = true;
                                // Setting the authentication params in cookies with expiry time of 30 mins.
                                var expiryDate = new Date();
                                expiryDate.setMinutes(expiryDate.getMinutes() + 30);
                                var apiAuthentication = { username: $scope.username, serial: $scope.serial, password: $scope.password };
                                document.cookie = "apiAuthentication=" + JSON.stringify(apiAuthentication) + "; expires=" + expiryDate;

                            } else {
                                $scope.connectionInfo = apiResponse.error || "Error validating credentials";
                                document.cookie = "apiAuthentication=";
                            }
                        }).catch(function (e) {
                            $scope.showLoader = false;
                            $scope.connectionInfo = "Error validating credentials";
                            document.cookie = "apiAuthentication=";
                        });
                    }
                };

                $scope.onGetPAComponents = function () {
                    if ($scope.pa.documentName === "") {
                        $scope.pa_error = "Enter valid document name";
                    } else {
                        $scope.pa.isdocumentLookupSuccess = false;
                        $scope.pa.isComponentSelected = false;
                        $scope.pa.componentid = "";
                        $scope.showLoader = true;
                        $scope.getPAComponents().then((apiResponse) => {
                            $scope.showLoader = false;
                            if (apiResponse.status === "Success") {
                                var components = apiResponse.response;
                                $scope.pa.componentsList = [];
                                for (var component in components) {
                                    var comp = {};
                                    comp.id = component;
                                    comp.name = components[component].category + " - " + components[component].name;
                                    $scope.pa.componentsList.push(comp);
                                }
                                $scope.pa.isdocumentLookupSuccess = true;
                            } else {
                                $scope.pa_error = apiResponse.error || "Error getting the components";
                            }

                            // Getting the options to be displayed for Frequency dropdown if paFrequencies is empty
                            if ($scope.paFrequencies.length === 0) {
                                $scope.getPAFrequencies().then((apiResponse) => {
                                    if (apiResponse.status === "Success") {
                                        for (var freqObj in apiResponse.response) {
                                            $scope.paFrequencies.push({ id: freqObj, name: apiResponse.response[freqObj].name });
                                        }
                                    } else {
                                        console.log(apiResponse.error);
                                    }
                                }).catch(function (e) {
                                    console.log(e);
                                });
                            }

                        }).catch(function (e) {
                            $scope.showLoader = false;
                            $scope.pa_error = "Error getting the components";
                        });
                    }
                };

                $scope.onGetSPARComponents = function () {
                    if ($scope.spar.documentName === "") {
                        $scope.spar_error = "Enter valid document name";
                    } else {
                        $scope.spar.isdocumentLookupSuccess = false;
                        $scope.spar.isComponentSelected = false;
                        $scope.spar.componentid = "";
                        $scope.showLoader = true;
                        $scope.getSPARComponents().then((apiResponse) => {
                            $scope.showLoader = false;
                            if (apiResponse.status === "Success") {
                                var components = apiResponse.response;
                                $scope.spar.componentsList = [];
                                for (var component in components) {
                                    var comp = {};
                                    comp.id = component;
                                    comp.name = components[component].category + " - " + components[component].name;
                                    $scope.spar.componentsList.push(comp);
                                }
                                $scope.spar.isdocumentLookupSuccess = true;
                            } else {
                                $scope.spar_error = apiResponse.error || "Error getting the components";
                            }

                            // Getting the options to be displayed for Frequency dropdown if sparFrequencies is empty
                            if ($scope.sparFrequencies.length === 0) {
                                $scope.getSPARFrequencies().then((apiResponse) => {
                                    if (apiResponse.status === "Success") {
                                        for (var freqObj in apiResponse.response) {
                                            $scope.sparFrequencies.push({ id: freqObj, name: apiResponse.response[freqObj].name });
                                        }
                                    } else {
                                        console.log(apiResponse.error);
                                    }
                                }).catch(function (e) {
                                    console.log(e);
                                });
                            }

                        }).catch(function (e) {
                            $scope.showLoader = false;
                            $scope.spar_error = "Error getting the components";
                        });
                    }
                };

                $scope.onGetVaultComponents = function () {
                    if ($scope.vault.documentName === "") {
                        $scope.vault_error = "Enter valid document name";
                    } else {
                        $scope.vault.isdocumentLookupSuccess = false;
                        $scope.vault.isComponentSelected = false;
                        $scope.vault.componentid = "";
                        $scope.showLoader = true;
                        $scope.getVaultComponents().then((apiResponse) => {
                            $scope.showLoader = false;
                            if (apiResponse.status === "Success") {
                                var components = apiResponse.response;
                                $scope.vault.componentsList = [];
                                for (var component in components) {
                                    var comp = {};
                                    comp.id = component;
                                    comp.name = components[component].category + " - " + components[component].name;
                                    $scope.vault.componentsList.push(comp);
                                }
                                $scope.vault.isdocumentLookupSuccess = true;

                            } else {
                                $scope.vault_error = apiResponse.error || "Error getting the components";
                            }

                            //Getting the options for frequency dropdown if the vaultFrequencies is empty.
                            if ($scope.vaultFrequencies.length === 0) {
                                $scope.getVaultFrequencies().then((apiResponse) => {
                                    if (apiResponse.status === "Success") {
                                        for (var freqObj in apiResponse.response) {
                                            $scope.vaultFrequencies.push({ id: freqObj, name: apiResponse.response[freqObj].name });
                                        }
                                    } else {
                                        console.log(apiResponse.error);
                                    }
                                }).catch(function (e) {
                                    console.log(e);
                                });
                            }

                        }).catch(function (e) {
                            $scope.showLoader = false;
                            $scope.vault_error = "Error getting the components";
                        });
                    }
                };

                $scope.onPAComponentSelected = function () {
                    $scope.pa.isComponentSelected = false;
                    $scope.showLoader = true;
                    $scope.getPAComponenetDetails().then((apiResponse) => {
                        if (apiResponse.status === "Success") {
                            $scope.showLoader = false;
                            $scope.pa.isComponentSelected = true;
                            var componentDetails = apiResponse.response;

                            //Setting component default dates.
                            $scope.pa.dates.startdate = componentDetails.dates.startdate;
                            $scope.pa.dates.enddate = componentDetails.dates.enddate;
                            $scope.pa.dates.frequency = componentDetails.dates.frequency;

                            //Setting component default accounts and benchmarks
                            if (componentDetails.accounts) {
                                $scope.pa.hasDefaultAccounts = true;
                                componentDetails.accounts.forEach((account) => {
                                    $scope.pa.addAccount(account.id, account.holdingsmode);
                                });
                            }
                            if (componentDetails.benchmarks) {
                                $scope.pa.hasDefaultBenchmarks = true;
                                componentDetails.benchmarks.forEach((benchmark) => {
                                    $scope.pa.addBenchmark(benchmark.id, benchmark.holdingsmode);
                                });
                            }

                        } else {
                            $scope.showLoader = false;
                            $scope.pa_error = apiResponse.error || "Error getting component details";
                        }
                    }).catch(function (e) {
                        $scope.showLoader = false;
                        $scope.pa_error = "Error getting component details";
                    });
                };

                $scope.onSPARComponentSelected = function () {
                    $scope.spar.isComponentSelected = true;
                };

                $scope.onVaultComponentSelected = function () {
                    $scope.vault.isComponentSelected = false;
                    $scope.showLoader = true;
                    $scope.getVaultComponenetDetails().then((apiResponse) => {

                        if (apiResponse.status === "Success") {
                            var componentDetails = apiResponse.response;

                            $scope.vault.dates.startdate = componentDetails.dates.startdate;
                            $scope.vault.dates.enddate = componentDetails.dates.enddate;
                            $scope.vault.dates.frequency = componentDetails.dates.frequency;

                            if (componentDetails.account) {
                                $scope.vault.account.id = componentDetails.account.id;
                                $scope.getVaultConfigurations().then((apiResponse) => {
                                    if (apiResponse.status === "Success") {
                                        var configurations = apiResponse.response;
                                        $scope.setVaultConfigurationSelectOptions(configurations);
                                    } else {
                                        $scope.vault_error = apiResponse.error || "Error getting configuration details";
                                    }
                                    $scope.showLoader = false;
                                }).catch(function (e) {
                                    $scope.showLoader = false;
                                    $scope.vault_error = "Error getting vault configuration details";
                                    console.log(e);
                                });
                            } else {
                                $scope.vault.account = {};
                                $scope.vault.configurationsList = [];
                                $scope.showLoader = false;
                            }
                            $scope.vault.isComponentSelected = true;
                        } else {
                            $scope.vault_error = apiResponse.error || "Error getting component details";
                        }
                    }).catch(function (e) {
                        console.log(e);
                        $scope.showLoader = false;
                        $scope.vault_error = "Error getting component details";
                    });
                };

                $scope.onGetVaultConfigurations = function () {
                    $scope.showLoader = true;
                    $scope.getVaultConfigurations().then((apiResponse) => {
                        if (apiResponse.status === "Success") {
                            var configurations = apiResponse.response;
                            $scope.setVaultConfigurationSelectOptions(configurations);
                        } else {
                            $scope.vault.configurationsList = [];
                            $scope.vault_error = apiResponse.error || "Error getting vault configuration details";
                        }
                        $scope.showLoader = false;
                    }).catch(function (e) {
                        $scope.showLoader = false;
                        $scope.vault.configurationsList = [];
                        console.log(e);
                        $scope.vault_error = "Error getting vault configuration details";
                    });
                };

                $scope.setVaultConfigurationSelectOptions = function (configurations) {
                    $scope.vault.configurationsList = [];
                    for (var config in configurations) {
                        var configObject = {};
                        configObject.id = config;
                        configObject.name = configurations[config].name;
                        $scope.vault.configurationsList.push(configObject);
                    }
                };

                $scope.onOKClicked = function () {

                    if ($scope.paArray.length === 0 && $scope.sparArray.length === 0 && $scope.vaultArray.length === 0) {
                        $scope.connectionInfo = "Please add at least one PA, SPAR, or Vault calculation before creating a connection";
                    } else {
                        $scope.showLoader = true;
                        $scope.getCalculationPostBody();
                        $scope.getTablesMetada().then((metadataResponse) => {
                            $scope.showLoader = false;
                            if (metadataResponse.status === "Success") {
                                for (var resultUrl in metadataResponse.response) {
                                    $scope.resultLocations.push(resultUrl);
                                    var tableDefinitionList = metadataResponse.response[resultUrl];
                                    tableDefinitionList.forEach(tableDefinition => {
                                        $scope.tablesMetadata.push(tableDefinition);
                                    });
                                }
                                //array of TableDefinition
                                $scope.tablesMetadataString = JSON.stringify($scope.tablesMetadata).replace(/\"/g, "\'");
                                $scope.resultLocationsString = JSON.stringify($scope.resultLocations).replace(/\"/g, "\'");

                                input.serverside.createNewConnection($scope.name, $scope.buildConnectionString(), $scope.getUsernameSerial(), $scope.password);
                                $scope.destroyComponent();
                            }
                            else {
                                // If the metadataResponse.status is Failure and still has value for metadataResponse.response, which means that the batch
                                // calculation is partially successful
                                if (metadataResponse.response) {
                                    $scope.metadataResponseOnPartialSuccess = metadataResponse;
                                    $scope.batchHasFailureCalcs = true;
                                    $scope.batchFailuresSummary = metadataResponse.error;
                                } else {
                                    $scope.connectionInfo = metadataResponse.error || "Error in creating calculation";
                                }
                            }
                        }).catch(function (e) {
                            $scope.showLoader = false;
                            $scope.connectionInfo = "Error in creating calculation";
                        });
                    }
                };

                $scope.onEditPostBodyClicked = function () {
                    $scope.isEditPostBody = true;
                    input.serverside.getConnection(input.instanceId).then((result) => {
                        var connectionString = result.qConnection.qConnectionString;
                        var postbody = $scope.getPostBodyFromConnectionString(connectionString).replace(/\'/g, "\"");
                        var postBodyJson = JSON.stringify(JSON.parse(postbody), undefined, 4);
                        document.getElementById('editConnTextArea').innerHTML = postBodyJson;
                    }).catch(function (e) {
                        console.log(e);
                    });
                };

                $scope.onUpdatePostBodyClicked = function () {
                    input.serverside.getConnection(input.instanceId).then((result) => {
                        var connectionString = result.qConnection.qConnectionString;
                        var updatedPostBody = document.getElementById('editConnTextArea').value.replace(/\"/g, "\'").replace(/\n/g, "");
                        var str = ";postBody=" + updatedPostBody + ";tablesMetadata=";
                        var updatedConnectionString = connectionString.replace(/;postBody=(.*);tablesMetadata=/g, str);
                        $scope.showLoader = true;
                        $scope.updateConnectionByReRunningCalculation(updatedConnectionString, updatedPostBody).then((result) => {
                            $scope.showLoader = false;
                            if (result)
                                $scope.destroyComponent();
                        }).catch(function (e) {
                            $scope.showLoader = false;
                            console.log(e);
                            $scope.connectionInfo = "Error in updating connection by re-running calculation";
                        });
                    }).catch(function (e) {
                        console.log(e);
                    });

                };

                $scope.onReRunCalculationClicked = function () {
                    input.serverside.getConnection(input.instanceId).then((result) => {
                        var connectionString = result.qConnection.qConnectionString;
                        var postBody = $scope.getPostBodyFromConnectionString(connectionString);
                        $scope.showLoader = true;
                        $scope.updateConnectionByReRunningCalculation(connectionString, postBody).then((result) => {
                            $scope.showLoader = false;
                            if (result)
                                $scope.destroyComponent();
                        }).catch(function (e) {
                            $scope.showLoader = false;
                            console.log(e);
                            $scope.connectionInfo = "Error in updating connection by re-running calculation";
                        });
                    });
                };

                $scope.onEscape = $scope.onCancelClicked = function () {
                    $scope.destroyComponent();
                };
                //** Event handlers - End **

                //** Helper methods for event handlers - Start **
                $scope.continueToEditCalculationInputs = function () {
                    $scope.batchHasFailureCalcs = false;
                    $scope.batchFailuresSummary = "";
                };

                $scope.createConnectionWithPartialSuccessCalcs = function () {

                    for (var resultUrl in $scope.metadataResponseOnPartialSuccess.response) {
                        $scope.resultLocations.push(resultUrl);
                        var tableDefinitionList = $scope.metadataResponseOnPartialSuccess.response[resultUrl];
                        tableDefinitionList.forEach(tableDefinition => {
                            $scope.tablesMetadata.push(tableDefinition);
                        });
                    }

                    //array of TableDefinition
                    $scope.tablesMetadataString = JSON.stringify($scope.tablesMetadata).replace(/\"/g, "\'");
                    $scope.resultLocationsString = JSON.stringify($scope.resultLocations).replace(/\"/g, "\'");
                    $scope.getCalculationPostBody();
                    input.serverside.createNewConnection($scope.name, $scope.buildConnectionString(), $scope.getUsernameSerial(), $scope.password);
                    $scope.destroyComponent();
                };

                $scope.getUsernameSerialFromConnectionString = function (connString) {
                    var connectionParamsArray = connString.split(";");
                    var parameter = connectionParamsArray.find(o => o.startsWith("apiusernameserial="));
                    if (parameter) {
                        return parameter.replace("apiusernameserial=", "");
                    }
                };

                $scope.getPasswordFromConnectionString = function (connString) {
                    var connectionParamsArray = connString.split(";");
                    var parameter = connectionParamsArray.find(o => o.startsWith("apipassword="));
                    if (parameter) {
                        return parameter.replace("apipassword=", "");
                    }
                };

                $scope.getPostBodyFromConnectionString = function (connString) {
                    var connectionParamsArray = connString.split(";");
                    var parameter = connectionParamsArray.find(o => o.startsWith("postBody="));
                    if (parameter) {
                        return parameter.replace("postBody=", "");
                    }
                };

                /*
                  Triggers the calculation with the postBody provided and update the connection string
                  with the obtained result locations and tables metadata information.
                  Parameters:
                       connectionString -> current connection string.
                       postBody -> post body string using which the new calculations has to be run.
                */
                $scope.updateConnectionByReRunningCalculation = function (connectionString, postBody) {

                    var usernameSerial = $scope.getUsernameSerialFromConnectionString(connectionString);
                    $scope.password = $scope.getPasswordFromConnectionString(connectionString);
                    $scope.username = usernameSerial.split("-")[0];
                    $scope.serial = usernameSerial.split("-")[1];
                    $scope.postbody = postBody;

                    return input.serverside.sendJsonRequest("getTablesMetada", usernameSerial, $scope.password, postBody).then((response) => {
                        var metadataResponse = JSON.parse(response.qMessage);
                        if (metadataResponse.status === "Success") {
                            for (var resultUrl in metadataResponse.response) {
                                $scope.resultLocations.push(resultUrl);
                                var tableDefinitionList = metadataResponse.response[resultUrl];
                                tableDefinitionList.forEach(tableDefinition => {
                                    $scope.tablesMetadata.push(tableDefinition);
                                });
                            }

                            $scope.tablesMetadataString = JSON.stringify($scope.tablesMetadata).replace(/\"/g, "\'");
                            $scope.resultLocationsString = JSON.stringify($scope.resultLocations).replace(/\"/g, "\'");

                            var updatedConnectionString = $scope.buildConnectionString();

                            input.serverside.modifyConnection(input.instanceId,
                                $scope.qvxSelectedEntrypoint.name,
                                updatedConnectionString,
                                $scope.provider,
                                false,
                                usernameSerial,
                                $scope.password);
                            return true;
                        } else {
                            //We are not updating the connection string with partial calculation success in Edit Connection flow.
                            $scope.connectionInfo = "Failed to re-run the calculation " + metadataResponse.error;
                            return false;
                        }
                    }).catch(function (e) {
                        console.log(e);
                        return false;
                    });
                };
                //** Helper methods for event handlers - End **

                init();
            }]
        };
    });